@@ -13,6 +13,25 @@ from parler.utils import get_language_settings
 if six.PY3:
     long = int
 
+try:
+    # In Django 1.6, a timeout of 0 seconds is accepted as valid input,
+    # and a sentinel value is used to denote the default timeout. Use that.
+    from django.core.cache.backends.base import DEFAULT_TIMEOUT
+except ImportError:
+    DEFAULT_TIMEOUT = 0
+
+
+class IsMissing(object):
+    # Allow _get_any_translated_model() to evaluate this as False.
+    def __nonzero__(self):
+        return False   # Python 2
+    def __bool__(self):
+        return False   # Python 3
+    def __repr__(self):
+        return "<IsMissing>"
+
+MISSING = IsMissing()  # sentinel value
+
 
 def get_object_cache_keys(instance):
     """
@@ -38,35 +57,43 @@ def get_translation_cache_key(translated_model, master_id, language_code):
     return 'parler.{0}.{1}.{2}.{3}'.format(translated_model._meta.app_label, translated_model.__name__, long(master_id), language_code)
 
 
-def get_cached_translation(instance, language_code, use_fallback=False):
+def get_cached_translation(instance, language_code=None, use_fallback=False):
     """
     Fetch an cached translation.
     """
-    if not appsettings.PARLER_ENABLE_CACHING or not instance.pk or instance._state.adding:
-        return None
+    if language_code is None:
+        language_code = instance.get_current_language()
 
-    key = get_translation_cache_key(instance._translations_model, instance.pk, language_code)
-    values = cache.get(key)
+    values = _get_cached_values(instance, language_code, use_fallback)
     if not values:
         return None
 
-    # Check for a stored fallback marker
-    if values.get('__FALLBACK__', False):
-        # Allow to return the fallback language instead.
-        if use_fallback:
-            lang_dict = get_language_settings(language_code)
-            if lang_dict['fallback'] != language_code:
-                return get_cached_translation(instance, lang_dict['fallback'], use_fallback=False)
-        return None
-
-    values['master'] = instance
-    values['language_code'] = language_code
     translation = instance._translations_model(**values)
     translation._state.adding = False
     return translation
 
 
-def get_cached_translated_field(instance, language_code, field_name):
+def get_cached_translated_field(instance, field_name, language_code=None, use_fallback=False):
+    """
+    Fetch an cached field.
+    """
+    if language_code is None:
+        language_code = instance.get_current_language()
+
+    # In django-parler 1.1 the order of the arguments was fixed, It used to be language_code, field_name
+    # This serves as detection against backwards incompatibility issues.
+    if len(field_name) <= 5 and len(language_code) > 5:
+        raise RuntimeError("Unexpected language code, did you swap field_name, language_code?")
+
+    values = _get_cached_values(instance, language_code, use_fallback)
+    if not values:
+        return None
+
+    # Allow older cached versions where the field didn't exist yet.
+    return values.get(field_name, None)
+
+
+def _get_cached_values(instance, language_code, use_fallback=False):
     """
     Fetch an cached field.
     """
@@ -78,11 +105,24 @@ def get_cached_translated_field(instance, language_code, field_name):
     if not values:
         return None
 
-    # Allow older cached versions where the field didn't exist yet.
-    return values.get(field_name, None)
+    # Check for a stored fallback marker
+    if values.get('__FALLBACK__', False):
+        # Internal trick, already set the fallback marker, so no query will be performed.
+        instance._translations_cache[language_code] = MISSING
+
+        # Allow to return the fallback language instead.
+        if use_fallback:
+            lang_dict = get_language_settings(language_code)
+            if lang_dict['fallback'] != language_code:
+                return _get_cached_values(instance, lang_dict['fallback'], use_fallback=False)
+        return None
+
+    values['master'] = instance
+    values['language_code'] = language_code
+    return values
 
 
-def _cache_translation(translation, timeout=0):
+def _cache_translation(translation, timeout=DEFAULT_TIMEOUT):
     """
     Store a new translation in the cache.
     """
@@ -100,11 +140,12 @@ def _cache_translation(translation, timeout=0):
     cache.set(key, values, timeout=timeout)
 
 
-def _cache_translation_needs_fallback(instance, language_code, timeout=0):
+
+def _cache_translation_needs_fallback(instance, language_code, timeout=DEFAULT_TIMEOUT):
     """
     Store the fact that a translation doesn't exist, and the fallback should be used.
     """
-    if not instance.pk or instance._state.adding:
+    if not appsettings.PARLER_ENABLE_CACHING or not instance.pk or instance._state.adding:
         return
 
     key = get_translation_cache_key(instance._translations_model, instance.pk, language_code)
