@@ -2,7 +2,9 @@ from django import forms
 from django.forms.models import ModelFormMetaclass, BaseInlineFormSet
 from django.utils.translation import get_language
 from django.utils import six
-from parler.models import TranslatableModel, TranslationDoesNotExist
+from parler.models import TranslationDoesNotExist
+from parler.utils import compat
+
 
 __all__ = (
     'TranslatableModelForm',
@@ -65,22 +67,26 @@ class TranslatableModelFormMixin(object):
             else:
                 self.language_code = current_language or get_language()
 
-        # Make sure the language code is set as early as possible (so it's active during clean() methods)
-        #self.instance.set_current_language(self.language_code)
-
-    def save(self, *args, **kwargs):
-        # Using args, kwargs to support custom parent arguments too.
+    def _post_clean(self):
+        # Copy the translated fields into the model
+        # Make sure the language code is set as early as possible (so it's active during most clean() methods)
         self.instance.set_current_language(self.language_code)
-        self.save_translated_fields(*args, **kwargs)
-        return super(TranslatableModelFormMixin, self).save(*args, **kwargs)
+        self.save_translated_fields()
 
-    def save_translated_fields(self, *args, **kwargs):
+        # Perform the regular clean checks, this also updates self.instance
+        super(TranslatableModelFormMixin, self)._post_clean()
+
+    def save_translated_fields(self):
         """
         Save all translated fields.
         """
         # Assign translated fields to the model (using the TranslatedAttribute descriptor)
         for field in self._get_translated_fields():
-            setattr(self.instance, field, self.cleaned_data[field])
+            try:
+                value = self.cleaned_data[field]
+            except KeyError:  # Field has a ValidationError
+                continue
+            setattr(self.instance, field, value)
 
     def _get_translated_fields(self):
         return [f_name for f_name in self._meta.model._translations_model.get_translated_fields() if f_name in self.fields]
@@ -171,7 +177,7 @@ def _get_model_form_field(model, name, formfield_callback=None, **kwargs):
     Utility to create the formfield from a model field.
     When a field is not editable, a ``None`` will be returned.
     """
-    field = model._meta.get_field_by_name(name)[0]
+    field = model._meta.get_field(name)
     if not field.editable:  # see fields_for_model() logic in Django.
         return None
 
@@ -186,12 +192,14 @@ def _get_model_form_field(model, name, formfield_callback=None, **kwargs):
     return formfield
 
 
-class TranslatableModelForm(six.with_metaclass(TranslatableModelFormMetaclass, TranslatableModelFormMixin), forms.ModelForm):
+class TranslatableModelForm(compat.with_metaclass(TranslatableModelFormMetaclass, TranslatableModelFormMixin, forms.ModelForm)):
     """
     The model form to use for translated models.
     """
     # six.with_metaclass does not handle more than 2 parent classes for django < 1.6
-    # so only one is wrapped within with_metaclass
+    # but we need all of them in django 1.7 to pass check admin.E016:
+    #       "The value of 'form' must inherit from 'BaseModelForm'"
+    # so we use our copied version in parler.utils.compat
 
 
 class TranslatableBaseInlineFormSet(BaseInlineFormSet):
