@@ -56,7 +56,7 @@ The manager and queryset objects of django-parler can work together with django-
 from __future__ import unicode_literals
 import django
 from django.conf import settings
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import models, router
 from django.db.models.base import ModelBase
 from django.db.models.fields.related import ReverseSingleRelatedObjectDescriptor
@@ -64,9 +64,10 @@ from django.utils.functional import lazy
 from django.utils.translation import get_language, ugettext, ugettext_lazy as _
 from django.utils import six
 from parler import signals
-from parler.cache import _cache_translation, _cache_translation_needs_fallback, _delete_cached_translation, get_cached_translation, _delete_cached_translations, get_cached_translated_field
+from parler.cache import MISSING, _cache_translation, _cache_translation_needs_fallback, _delete_cached_translation, get_cached_translation, _delete_cached_translations, get_cached_translated_field
 from parler.fields import TranslatedField, LanguageCodeDescriptor, TranslatedFieldDescriptor
 from parler.managers import TranslatableManager
+from parler.utils import compat
 from parler.utils.i18n import normalize_language_code, get_language_settings, get_language_title
 import sys
 import logging
@@ -185,13 +186,14 @@ class TranslatedFields(object):
         assert cls._translations_field == name
 
 
-
 class TranslatableModel(models.Model):
     """
     Base model class to handle translations.
 
     All translatable fields will appear on this model, proxying the calls to the :class:`TranslatedFieldsModel`.
     """
+    class Meta:
+        abstract = True
 
     # Consider these fields "protected" or "internal" attributes.
     # Not part of the public API, but used internally in the class hierarchy.
@@ -203,10 +205,6 @@ class TranslatableModel(models.Model):
     # change the default manager to the translation manager
     objects = TranslatableManager()
 
-    class Meta:
-        abstract = True
-
-
     def __init__(self, *args, **kwargs):
         # Still allow to pass the translated fields (e.g. title=...) to this function.
         translated_kwargs = {}
@@ -219,12 +217,12 @@ class TranslatableModel(models.Model):
                 except KeyError:
                     pass
 
-        # Run original Django model __init__
-        super(TranslatableModel, self).__init__(*args, **kwargs)
-
         self._translations_cache = {}
         self._current_language = normalize_language_code(current_language or get_language())  # What you used to fetch the object is what you get.
 
+        # Run original Django model __init__
+        super(TranslatableModel, self).__init__(*args, **kwargs)
+
         # Assign translated args manually.
         if translated_kwargs:
             translation = self._get_translated_model(auto_create=True)
@@ -271,8 +269,14 @@ class TranslatableModel(models.Model):
         try:
             # Check the local cache directly, and the answer is known.
             # NOTE this may also return newly auto created translations which are not saved yet.
-            return self._translations_cache[language_code] is not None
+            return self._translations_cache[language_code] is not MISSING
         except KeyError:
+            # Try to fetch from the cache first.
+            # If the cache returns the fallback, it means the original does not exist.
+            object = get_cached_translation(self, language_code, use_fallback=True)
+            if object is not None:
+                return object.language_code == language_code
+
             try:
                 # Fetch from DB, fill the cache.
                 self._get_translated_model(language_code, use_fallback=False, auto_create=False)
@@ -286,10 +290,11 @@ class TranslatableModel(models.Model):
         """
         Return the language codes of all translated variations.
         """
-        qs = self._get_translated_queryset()
-        if qs._prefetch_done:
-            return sorted(obj.language_code for obj in qs)
+        prefetch = self._get_prefetched_translations()
+        if prefetch is not None:
+            return sorted(obj.language_code for obj in prefetch)
         else:
+            qs = self._get_translated_queryset()
             return qs.values_list('language_code', flat=True).order_by('language_code')
 
 
@@ -308,18 +313,18 @@ class TranslatableModel(models.Model):
             object = self._translations_cache[language_code]
 
             # If cached object indicates the language doesn't exist, need to query the fallback.
-            if object is not None:
+            if object is not MISSING:
                 return object
         except KeyError:
             # 2. No cache, need to query
             # Check that this object already exists, would be pointless otherwise to check for a translation.
             if not self._state.adding and self.pk:
-                qs = self._get_translated_queryset()
-                if qs._prefetch_done:
+                prefetch = self._get_prefetched_translations()
+                if prefetch is not None:
                     # 2.1, use prefetched data
                     # If the object is not found in the prefetched data (which contains all translations),
                     # it's pointless to check for memcached (2.2) or perform a single query (2.3)
-                    for object in qs:
+                    for object in prefetch:
                         if object.language_code == language_code:
                             self._translations_cache[language_code] = object
                             _cache_translation(object)  # Store in memcached
@@ -330,13 +335,17 @@ class TranslatableModel(models.Model):
                     if object is not None:
                         # Track in local cache
                         if object.language_code != language_code:
-                            self._translations_cache[language_code] = None  # Set fallback marker
+                            self._translations_cache[language_code] = MISSING  # Set fallback marker
                         self._translations_cache[object.language_code] = object
                         return object
+                    elif self._translations_cache.get(language_code, None) is MISSING:
+                        # If get_cached_translation() explicitly set the "does not exist" marker,
+                        # there is no need to try a database query.
+                        pass
                     else:
                         # 2.3, fetch from database
                         try:
-                            object = qs.get(language_code=language_code)
+                            object = self._get_translated_queryset().get(language_code=language_code)
                         except self._translations_model.DoesNotExist:
                             pass
                         else:
@@ -362,13 +371,14 @@ class TranslatableModel(models.Model):
         fallback_msg = None
         lang_dict = get_language_settings(language_code)
 
-        if use_fallback and (lang_dict['fallback'] != language_code):
+        if language_code not in self._translations_cache:
             # Explicitly set a marker for the fact that this translation uses the fallback instead.
             # Avoid making that query again.
-            self._translations_cache[language_code] = None  # None value is the marker.
+            self._translations_cache[language_code] = MISSING  # None value is the marker.
             if not self._state.adding or self.pk:
                 _cache_translation_needs_fallback(self, language_code)
 
+        if lang_dict['fallback'] != language_code and use_fallback:
             # Jump to fallback language, return directly.
             # Don't cache under this language_code
             try:
@@ -394,17 +404,17 @@ class TranslatableModel(models.Model):
             try:
                 return self._translations_cache.get(self._current_language, None) \
                     or self._translations_cache.get(self.get_fallback_language(), None) \
-                    or next(t for t in six.itervalues(self._translations_cache) if t if not None)  # Skip fallback markers.
+                    or next(t for t in six.itervalues(self._translations_cache) if t if not MISSING)  # Skip fallback markers.
             except StopIteration:
                 pass
 
         try:
             # Use prefetch if available, otherwise perform separate query.
-            qs = self._get_translated_queryset()
-            if qs._prefetch_done:
-                translation = list(qs)[0]
+            prefetch = self._get_prefetched_translations()
+            if prefetch is not None:
+                translation = prefetch[0]  # Already a list
             else:
-                translation = qs[0]
+                translation = self._get_translated_queryset()[0]
         except IndexError:
             return None
         else:
@@ -427,6 +437,17 @@ class TranslatableModel(models.Model):
             return accessor.get_query_set()
 
 
+    def _get_prefetched_translations(self):
+        """
+        Return the queryset with prefetch results.
+        """
+        try:
+            # Read the list directly, avoid QuerySet construction.
+            # Accessing self._get_translated_queryset()._prefetch_done is more expensive.
+            return self._prefetched_objects_cache[self._translations_field]
+        except (AttributeError, KeyError):
+            return None
+
     def save(self, *args, **kwargs):
         super(TranslatableModel, self).save(*args, **kwargs)
         self.save_translations(*args, **kwargs)
@@ -437,6 +458,31 @@ class TranslatableModel(models.Model):
         super(TranslatableModel, self).delete(using)
 
 
+    def validate_unique(self, exclude=None):
+        """
+        Also validate the unique_together of the translated model.
+        """
+        # This is called from ModelForm._post_clean() or Model.full_clean()
+        errors = {}
+        try:
+            super(TranslatableModel, self).validate_unique(exclude=exclude)
+        except ValidationError as e:
+            errors = e.message_dict  # Django 1.5 + 1.6 compatible
+
+        translations = self._translations_cache.values()
+        for translation in translations:
+            if translation is MISSING:  # Skip fallback markers
+                continue
+
+            try:
+                translation.validate_unique(exclude=exclude)
+            except ValidationError as e:
+                errors.update(e.message_dict)
+
+        if errors:
+            raise ValidationError(errors)
+
+
     def save_translations(self, *args, **kwargs):
         """
         The method to save all translations.
@@ -452,7 +498,7 @@ class TranslatableModel(models.Model):
         # Save all translated objects which were fetched.
         # This also supports switching languages several times, and save everything in the end.
         for translation in translations:
-            if translation is None:  # Skip fallback markers
+            if translation is MISSING:  # Skip fallback markers
                 continue
 
             self.save_translation(translation, *args, **kwargs)
@@ -488,26 +534,20 @@ class TranslatableModel(models.Model):
         Also consider using ``field = TranslatedField(any_language=True)`` in the model itself,
         to make this behavior the default for the given field.
         """
-        # By default, query via descriptor (TranslatedFieldDescriptor)
-        # which also attempts the fallback language if configured to do so.
-        tr_model = self
-
         # Extra feature: query a single field from a other translation.
         if language_code and language_code != self._current_language:
-            # Try to fetch a cached value first.
-            value = get_cached_translated_field(self, language_code, field)
-            if value is not None:
-                return value
-
             try:
                 tr_model = self._get_translated_model(language_code)
+                return getattr(tr_model, field)
             except TranslationDoesNotExist:
-                pass  # Use 'self'
-
-        try:
-            return getattr(tr_model, field)
-        except TranslationDoesNotExist:
-            pass
+                pass
+        else:
+            # By default, query via descriptor (TranslatedFieldDescriptor)
+            # which also attempts the fallback language if configured to do so.
+            try:
+                return getattr(self, field)
+            except TranslationDoesNotExist:
+                pass
 
         if any_language:
             translation = self._get_any_translated_model()
@@ -579,7 +619,7 @@ def _validate_master(new_class):
     return shared_model
 
 
-class TranslatedFieldsModel(six.with_metaclass(TranslatedFieldsModelBase, models.Model)):
+class TranslatedFieldsModel(compat.with_metaclass(TranslatedFieldsModelBase, models.Model)):
     """
     Base class for the model that holds the translated fields.
     """
