@@ -1,12 +1,13 @@
+from __future__ import print_function
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.core.management import call_command
 from django.contrib.sites.models import Site
 from django.db.models import loading
-from django.template.loaders import app_directories
 from django.test import TestCase
 from django.utils.importlib import import_module
 import os
+from parler import appsettings
 
 
 class AppTestCase(TestCase):
@@ -21,12 +22,13 @@ class AppTestCase(TestCase):
 
     @classmethod
     def setUpClass(cls):
+        from django.template.loaders import app_directories  # late import, for django 1.7
         if cls.install_apps:
             # When running this app via `./manage.py test fluent_pages`, auto install the test app + models.
             run_syncdb = False
             for appname in cls.install_apps:
                 if appname not in settings.INSTALLED_APPS:
-                    print 'Adding {0} to INSTALLED_APPS'.format(appname)
+                    print('Adding {0} to INSTALLED_APPS'.format(appname))
                     settings.INSTALLED_APPS = (appname,) + tuple(settings.INSTALLED_APPS)
                     run_syncdb = True
 
@@ -44,3 +46,8 @@ class AppTestCase(TestCase):
         # 1.4 does not create site automatically with the defined SITE_ID, 1.3 does.
         Site.objects.get_or_create(id=settings.SITE_ID, defaults=dict(domain='django.localhost', name='django at localhost'))
         cls.user, _ = User.objects.get_or_create(is_superuser=True, is_staff=True, username="admin")
+
+        # Be supportive for other project settings too.
+        cls.conf_fallback = appsettings.PARLER_LANGUAGES['default']['fallback'] or 'en'
+        cls.other_lang1 = next(x for x, _ in settings.LANGUAGES if x != cls.conf_fallback)
+        cls.other_lang2 = next(x for x, _ in settings.LANGUAGES if x not in (cls.conf_fallback, cls.other_lang1))
